@@ -1,11 +1,8 @@
 from cliff.show import ShowOne
 
-from enough.common.service import service_factory
-
-
-def set_common_options(parser):
-    parser.add_argument('--driver', default='openstack')
-    return parser
+from enough import settings
+from enough.common import options
+from enough.common import Enough
 
 
 class Create(ShowOne):
@@ -15,13 +12,13 @@ class Create(ShowOne):
         parser = super().get_parser(prog_name)
         parser.add_argument('name')
         parser.add_argument('--playbook', default='enough-playbook.yml')
-        return set_common_options(parser)
+        return options.set_options(parser)
 
     def take_action(self, parsed_args):
         args = vars(self.app.options)
         args.update(vars(parsed_args))
-        service = service_factory(**args)
-        r = service.create_or_update()
+        e = Enough(settings.CONFIG_DIR, settings.SHARE_DIR, **args)
+        r = e.service.create_or_update()
         columns = ('name',)
         data = (r['fqdn'],)
         return (columns, data)
