@@ -23,7 +23,8 @@ class OpenStackBase(object):
     INTERNAL_NETWORK = 'internal'
     INTERNAL_NETWORK_CIDR = '10.20.30.0/24'
 
-    def __init__(self, config_dir, config_file):
+    def __init__(self, config_dir, config_file, **kwargs):
+        self.args = kwargs
         self.config_dir = config_dir
         self.config_file = config_file
         self.o = sh.openstack.bake(
@@ -37,13 +38,13 @@ class OpenStackBase(object):
 
 class Stack(OpenStackBase):
 
-    def __init__(self, config_dir, config_file, definition=None):
-        super().__init__(config_dir, config_file)
+    def __init__(self, config_dir, config_file, definition=None, **kwargs):
+        super().__init__(config_dir, config_file, **kwargs)
         log.info(f'OS_CLIENT_CONFIG_FILE={self.config_file}')
         self.definition = definition
 
     def get_template(self):
-        return f'{settings.SHARE_DIR}/molecule/infrastructure/template-host.yaml'
+        return f'{settings.SHARE_DIR}/playbooks/infrastructure/template-host.yaml'
 
     def set_public_key(self, path):
         self.public_key = open(path).read().strip()
@@ -91,11 +92,14 @@ class Stack(OpenStackBase):
         line = s.makefile("rb").readline()
         assert line.startswith(b'SSH-')
 
-    def create_or_update(self):
+    def create_or_update(self, return_on_create=False):
         if self.definition['name'] not in self.list():
             self.create()
+            if return_on_create:
+                return None
         info = self.update()
-        self.wait_for_ssh(info['ipv4'], info['port'])
+        if not self.only_internal() or self.args.get('route_to_internal', True):
+            self.wait_for_ssh(info['ipv4'], info['port'])
         Hosts(self.config_dir).create_or_update(
             self.definition['name'], info['ipv4'], info['port'])
         return info
@@ -120,6 +124,9 @@ class Stack(OpenStackBase):
         self.connect_internal_network()
         return r
 
+    def only_internal(self):
+        return self.definition.get('network', 'Ext-Net') != 'Ext-Net'
+
     def create(self):
         try:
             log.info('create on network Ext-Net')
@@ -128,39 +135,48 @@ class Stack(OpenStackBase):
             log.info('retry create on network Ext-Net')
             # retry once because there is no way to increase the timeout and create fails often
             info = self._create_or_update('update', 'Ext-Net')
-        if self.definition.get('network', 'Ext-Net') != 'Ext-Net':
+        if self.only_internal():
             log.info(f'remove network Ext-Net')
             # wait for the interface to settle before removing it
             self.wait_for_ssh(info['ipv4'], info['port'])
             server = self.definition['name']
             self.o.server.remove.network(server, 'Ext-Net')
 
-    def delete(self):
+    def delete_no_wait(self):
         name = self.definition['name']
         if name not in self.list():
-            return
+            return False
 
-        self.o.stack.delete('--yes', '--wait', name)
+        self.o.stack.delete('--yes', name)
+        return True
+
+    def delete_wait(self):
+        name = self.definition['name']
 
         @retry(AssertionError, 9)
         def wait_is_deleted():
             assert name not in self.list(), f'{name} deletion in progress'
         wait_is_deleted()
 
-        Hosts(self.config_dir).delete(self.definition['name'])
+        Hosts(self.config_dir).delete(name)
 
         network = self.definition.get('internal_network', OpenStackBase.INTERNAL_NETWORK)
         o = OpenStack(self.config_dir, self.config_file)
         o.network_delete_if_not_used(network)
 
+    def delete(self):
+        if self.delete_no_wait():
+            self.delete_wait()
+
 
 class Heat(OpenStackBase):
 
     def get_stack_definitions(self, share_dir=settings.SHARE_DIR):
         args = ['-i', f'{share_dir}/inventory',
-                '-i', f'{self.config_dir}/inventory',
-                '-i', f'{self.config_dir}/development-inventory',
-                '--vars', '--list']
+                '-i', f'{self.config_dir}/inventory']
+        if self.args.get('inventory'):
+            args.extend([f'--inventory={i}' for i in self.args['inventory']])
+        args.extend(['--vars', '--list'])
         password_file = f'{self.config_dir}.pass'
         if os.path.exists(password_file):
             args.extend(['--vault-password-file', password_file])
@@ -175,8 +191,7 @@ class Heat(OpenStackBase):
             'port': h.get('ansible_port', '22'),
             'flavor': h.get('openstack_flavor', 's1-2'),
             'network': h.get('openstack_network', 'Ext-Net'),
-            'internal_network': h.get('openstack_internal_network',
-                                      OpenStackBase.INTERNAL_NETWORK),
+            'internal_network': OpenStackBase.INTERNAL_NETWORK,
             'internal_network_cidr': h.get('openstack_internal_network_cidr',
                                            OpenStackBase.INTERNAL_NETWORK_CIDR),
         }
@@ -205,21 +220,38 @@ class Heat(OpenStackBase):
 
     def create_or_update(self, names, public_key):
         r = {}
+        created = []
+        #
+        # Launch the creation of all stacks in // and do not wait for them to complete
+        #
         for name in names:
-            s = Stack(self.config_dir, self.config_file, self.get_stack_definition(name))
+            s = Stack(self.config_dir, self.config_file, self.get_stack_definition(name),
+                      **self.args)
             s.set_public_key(public_key)
+            info = s.create_or_update(return_on_create=True)
+            if info:
+                r[name] = info
+            else:
+                created.append(name)
+        #
+        # Verify all stacks previously launched complete as expected
+        #
+        for name in created:
+            s = Stack(self.config_dir, self.config_file, self.get_stack_definition(name),
+                      **self.args)
+            s.set_public_key(public_key)
+            info = s.create_or_update(return_on_create=True)
             r[name] = s.create_or_update()
         return r
 
     def create_test_subdomain(self, domain):
-        # exclusively when running from molecule
-        assert os.path.exists('molecule.yml')
         d = f"{self.config_dir}/inventory/group_vars/all"
         assert os.path.exists(d)
-        if 'bind-host' not in Stack(self.config_dir, self.config_file).list():
+        if 'bind-host' not in Stack(self.config_dir, self.config_file, **self.args).list():
             return None
         h = Heat(self.config_dir, self.config_file)
-        s = Stack(self.config_dir, self.config_file, h.get_stack_definition('bind-host'))
+        s = Stack(self.config_dir, self.config_file, h.get_stack_definition('bind-host'),
+                  **self.args)
         s.set_public_key(f'{self.config_dir}/infrastructure_key.pub')
         bind_host = s.create_or_update()
 
@@ -262,6 +294,7 @@ class OpenStack(OpenStackBase):
 
     @retry(OpenStackLeftovers, tries=7)
     def destroy_everything(self, prefix):
+        log.info('destroy_everything acting')
         leftovers = []
 
         def delete_snapshots():
@@ -349,14 +382,28 @@ class OpenStack(OpenStackBase):
         if not self.subnet_exists(name):
             self.o.subnet.create('--subnet-range', cidr, '--network', name, name)
 
+    def subnet_update_dns(self, name, *dns):
+        args = [f'--dns-nameserver={ip}' for ip in dns]
+        args.append(name)
+        self.o.subnet.set('--no-dns-nameserver', *args)
+
     def network_and_subnet_create(self, name, cidr):
         self.network_create(name)
         self.subnet_create(name, cidr)
 
     def server_connected_to_network(self, server, network):
-        port = self.o.port.list('--server', server, '--network', network,
-                                '--format=value', '-c', 'ID')
-        return port.strip() != ''
+        return self.server_ip_in_network(server, network)
+
+    def server_ip_in_network(self, server, network):
+        info = self.o.port.list('--server', server, '--network', network,
+                                '--format=value', '-c', 'Fixed IP Addresses')
+        info = info.strip()
+        if info == '':
+            return None
+        pattern = r"ip_address='(.*?)'"
+        m = re.search(pattern, info)
+        assert m, f're.search({pattern}, {info})'
+        return m.group(1)
 
     def backup_date(self):
         return datetime.datetime.today().strftime('%Y-%m-%d')
