@@ -13,16 +13,49 @@ log = logging.getLogger(__name__)
 
 class Service(object):
 
-    def __init__(self, config_dir, share_dir):
+    def __init__(self, config_dir, share_dir, **kwargs):
         self.config_dir = config_dir
         self.share_dir = share_dir
+        self.ansible = ansible_utils.Ansible(
+            self.config_dir, self.share_dir, kwargs.get('inventory', []))
+        self.set_service2hosts()
+        self.update_vpn_dependencies()
+
+    def set_service2hosts(self):
         suffix = '-service-hosts'
-        groups = ansible_utils.Ansible(config_dir, share_dir).get_groups()
+        groups = self.ansible.get_groups()
         self.service2hosts = {
             name.replace(suffix, ''): hosts for name, hosts in groups.items()
             if name.endswith(suffix)
         }
 
+    def update_vpn_dependencies(self):
+        hosts = self.ansible.ansible_inventory()['_meta']['hostvars'].keys()
+        internal_hosts = set(self.hosts_with_internal_network(hosts))
+        if not internal_hosts:
+            return
+        assert 'openvpn' in self.service2hosts
+        openvpn_hosts = set(self.service2hosts['openvpn'])
+        for service in self.service2hosts.keys():
+            if service == 'openvpn':
+                continue
+            hosts = set(self.service2hosts[service])
+            if internal_hosts & hosts:
+                self.service2hosts[service] = list(internal_hosts | openvpn_hosts)
+
+    def add_vpn_hosts_if_needed(self, hosts):
+        internal_hosts = set(self.hosts_with_internal_network(hosts))
+        if not internal_hosts:
+            return hosts
+        assert 'openvpn' in self.service2hosts
+        openvpn_hosts = set(self.service2hosts['openvpn'])
+        hosts = set(hosts)
+        return list(hosts | openvpn_hosts)
+
+    def hosts_with_internal_network(self, hosts):
+        networks = self.ansible.get_variable_hosts('encrypted_device', 'openstack_network', *hosts)
+        return [host for (host, network) in networks.items() if network == 'internal']
+
     def service_from_host(self, host):
         found = None
         hosts_count = 0
@@ -47,7 +80,7 @@ class ServiceOpenStack(Service):
         pass
 
     def __init__(self, config_dir, share_dir, **kwargs):
-        super().__init__(config_dir, share_dir)
+        super().__init__(config_dir, share_dir, **kwargs)
         self.args = kwargs
         self.dotenough = dotenough.DotEnoughOpenStack(config_dir, self.args['domain'])
         self.dotenough.ensure()
